@@ -2,7 +2,7 @@ use strict;
 use warnings;
 package MetaCPAN::Client::Scroll;
 # ABSTRACT: A MetaCPAN::Client scroller
-$MetaCPAN::Client::Scroll::VERSION = '2.031001';
+$MetaCPAN::Client::Scroll::VERSION = '2.032000';
 use Moo;
 use Carp;
 use Ref::Util qw< is_hashref >;
@@ -54,9 +54,9 @@ has _buffer => (
     default  => sub { [] },
 );
 
-has _done => (
+has _remaining => (
     is       => 'rw',
-    isa      => Bool,
+    isa      => Int,
     default  => sub { 0 },
 );
 
@@ -99,6 +99,7 @@ sub BUILDARGS {
     $args{_id}     = $content->{_scroll_id};
     $args{total}   = $content->{hits}{total};
     $args{_buffer} = $content->{hits}{hits};
+    $args{_remaining} = $content->{hits}{total};
 
     $args{aggregations} = $content->{aggregations}
         if $content->{aggregations} and is_hashref( $content->{aggregations} );
@@ -109,21 +110,27 @@ sub BUILDARGS {
 sub next {
     my $self   = shift;
     my $buffer = $self->_buffer;
+    my $remaining = $self->_remaining;
 
-    # We're exhausted and will do no more.
-    return if $self->_done;
+    if (!$remaining) {
+        # We're exhausted and will do no more.
+        return undef;
+    }
+    elsif (!@$buffer) {
+        # Refill the buffer if it's empty.
+        @$buffer = @{ $self->_fetch_next };
 
-    # Refill the buffer if it's empty.
-    @$buffer = @{ $self->_fetch_next }
-        unless @$buffer;
+        if (!@$buffer) {
+            # we weren't able to refill for some reason
+            $self->_remaining(0);
+            return undef;
+        }
+    }
 
-    # Grab the next result from the buffer.  If there's no result, then that's
-    # all, folks!
-    my $next = shift @$buffer;
-
-    $self->_done(1) unless $next;
-
-    return $next;
+    # One less result to return
+    $self->_remaining($remaining - 1);
+    # Return the next result
+    return shift @$buffer;
 }
 
 sub _fetch_next {
@@ -143,11 +150,21 @@ sub _fetch_next {
 }
 
 sub DEMOLISH {
-    my $self = shift;
+    my ( $self, $gd ) = @_;
+    return
+        if $gd;
+    my $ua = $self->ua
+        or return;
+    my $base_url = $self->base_url
+        or return;
+    my $id = $self->_id
+        or return;
+    my $time = $self->time
+        or return;
 
-    $self->ua->delete(
-        sprintf( '%s/_search/scroll?scroll=%s', $self->base_url, $self->time ),
-        { content => $self->_id }
+    $ua->delete(
+        sprintf( '%s/_search/scroll?scroll=%s', $base_url, $time ),
+        { content => $id }
     );
 }
 
@@ -165,7 +182,7 @@ MetaCPAN::Client::Scroll - A MetaCPAN::Client scroller
 
 =head1 VERSION
 
-version 2.031001
+version 2.032000
 
 =head1 METHODS
 
